# Generated by roxygen2: do not edit by hand

export("%>%")
export(abba_get_batch_log)
export(abba_get_batch_status)
export(abba_get_job_log)
export(abba_get_job_status)
export(abba_get_k8s_batch_log_local)
export(abba_get_k8s_batch_status_local)
export(abba_get_k8s_job_log_local)
export(abba_get_k8s_job_status_local)
export(abba_rslauncher_get_job_status_local)
export(abba_rslauncher_get_job_succeeded_local)
export(abba_rslauncher_submit_job_local)
export(abba_rslauncher_submit_logrx_job_local)
export(abba_rslauncher_watch_job_local)
export(abba_slurm_get_job_log)
export(abba_slurm_get_job_status)
export(abba_slurm_get_job_succeeded)
export(abba_slurm_submit_job)
export(abba_slurm_watch_job)
export(abba_submit_and_get_log)
export(abba_submit_batch)
export(abba_submit_batch_and_get_results)
export(abba_submit_job)
export(abba_submit_k8s_job_and_poll_local)
export(abba_submit_k8s_job_local)
export(abba_wait_for_batch_log)
export(abba_wait_for_job_log)
export(abba_watch_k8s_batch_local)
export(abba_watch_k8s_job_local)
export(calculate_run_group)
export(create_batch_api)
export(create_batch_job)
export(submit_k8s_yaml)
importFrom(magrittr,"%>%")
