## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(abba)

## ----submit_job, eval=FALSE---------------------------------------------------
# abba_slurm_submit_job(
#   "/home/mike.stackhouse/repos/abba/test_programs/test_program.R",
#   log_path = "/home/mike.stackhouse/test_logs"
#   )
# # [1] "38"

## ----get_status, eval=FALSE---------------------------------------------------
# abba_slurm_get_job_status("57")
# # 57
# # "COMPLETED"

## ----watch_job, eval=FALSE----------------------------------------------------
# abba_slurm_get_job_log("38")
# # [[1]]
# # [1] "[1] 4"                   "Warning message:"        "This is a test warning " "La-di-da"

## ----submit batch, eval=FALSE-------------------------------------------------
# abba_submit_batch(list("/home/mike.stackhouse/repos/abba/test_programs/test_program.R", "/home/mike.stackhouse/repos/abba/test_programs/test_program.R"),
#                   submit_func=abba_slurm_submit_job,
#                   wait_func=abba_slurm_watch_job,
#                   succeed_func=abba_slurm_get_job_succeeded,
#                   log_path="/home/mike.stackhouse/test_logs")
# # Submitting program:
# # 	/home/mike.stackhouse/repos/abba/test_programs/test_program.R
# # Submitting program:
# # 	/home/mike.stackhouse/repos/abba/test_programs/test_program.R
# # /home/mike.stackhouse/repos/abba/test_programs/test_program.R /home/mike.stackhouse/repos/abba/test_programs/test_program.R
#                                                           "48"                                                           "49"

