## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(abba)

## ----create_api, eval=FALSE---------------------------------------------------
# create_batch_api("path/to/api_directory")
# # Batch API file created at path/to/api_directory/plumber.R

## ----options, eval=FALSE------------------------------------------------------
# options(
#   abba.lower.cpu.limit=0.5,
#   abba.cpu.limit = 2,
#   abba.lower.memory.limit='128M',
#   abba.memory.limit='1G',
#   abba.permitted.containers = c("registry.com/openval_4.2.1_focal:2023.09.0.02",
#                                 "registry.com/openval_4.2.1_focal:latest",
#                                 "registry.com/openval_base_4.3.2_focal:2024.03.01",
#                                 "registry.com/openval_base_4.3.2_focal:latest",
#                                 "registry.com/openval-dev-focal:latest"),
#   abba.default.container = "registry.com/openval_4.2.1_focal:2023.09.0.02"
#   )

