------
title: "{abba} Job Template"
author: "`r Sys.info()['user']`"
date: "`r Sys.Date()`"
output: html_document
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

```{r test}
abba::abba_submit_and_get_log(
  "/path/to/file.R",
  container = "<container_image_name>"
  )
```
