# Error generates properly when container name is invalid

    The container bad_name is not an permitted image. Permitted images are:
    	- path.io/container-1.0.0
    	- path.io/container2-1.0.0
    	- path.io/container3-1.0.0

