% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/submit_job_slurm.R
\name{abba_slurm_watch_job}
\alias{abba_slurm_watch_job}
\title{Watch SLURM job, periodically polling its execution status.}
\usage{
abba_slurm_watch_job(
  job_id = "",
  poll_interval_seconds = 3,
  timeout_seconds = 300,
  ...
)
}
\arguments{
\item{job_id}{job ID that was specified when submitting jobs}

\item{poll_interval_seconds}{Time interval for polling job status in seconds}

\item{timeout_seconds}{Total time to wait before timeout in seconds}

\item{...}{other positional/keyword arguments that will be ignored}
}
\value{
a list of job ID(s) and status(es)
}
\description{
Watch SLURM job, periodically polling its execution status.
}
\examples{
\dontrun{
result <- abba_slurm_watch_job("5195", 10, 3000)
}

}
