% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/submit_job_slurm.R
\name{abba_slurm_get_job_status}
\alias{abba_slurm_get_job_status}
\title{Return descriptive job status for slurm jobs}
\usage{
abba_slurm_get_job_status(job_ids, ...)
}
\arguments{
\item{job_ids}{a list/vector of Slurm job IDs}

\item{...}{other positional/keyword arguments that will be ignored}
}
\value{
a named character vector with job statuses as values and job IDs as names.
}
\description{
Return descriptive job status for slurm jobs
}
\examples{
\dontrun{
job_statuses <- slurm_get_job_status(c('job-id-1', 'job-id-2'))
}
}
