% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/submit_job_workbench.R
\name{abba_rslauncher_watch_job_local}
\alias{abba_rslauncher_watch_job_local}
\title{Periodically poll Workbench jobs for status and return their IDs when all job
statuses arrive at 'Finished' state}
\usage{
abba_rslauncher_watch_job_local(
  job_ids,
  poll_interval_seconds = 1,
  timeout_seconds = 300,
  ...
)
}
\arguments{
\item{job_ids}{a list/vector of Workbench job IDs}

\item{poll_interval_seconds}{how often job statuses should be updated}

\item{timeout_seconds}{maximum amount of time in seconds after which job IDs will be
returned regardless of job statuses}

\item{...}{other positional/keyword arguments that will be ignored}
}
\value{
a vector of job IDs
}
\description{
Periodically poll Workbench jobs for status and return their IDs when all job
statuses arrive at 'Finished' state
}
\examples{
\dontrun{
job_statuses <- abba_rslauncher_watch_job_local(c('job-id-1', 'job-id-2'))
}
}
