% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/submit_job_workbench.R
\name{abba_rslauncher_get_job_succeeded_local}
\alias{abba_rslauncher_get_job_succeeded_local}
\title{Check whether Workbench jobs have been fully executed.}
\usage{
abba_rslauncher_get_job_succeeded_local(job_ids, ...)
}
\arguments{
\item{job_ids}{a list/vector of Workbench job IDs}

\item{...}{other positional/keyword arguments that will be ignored}
}
\value{
a named boolean vector. FALSE value indicates that job did not fully execute
}
\description{
Check whether Workbench jobs have been fully executed.
}
\examples{
\dontrun{
job_statuses <- abba_rslauncher_get_job_succeeded_local(c('job-id-1', 'job-id-2'))
}
}
