% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/submit_job_k8s.R
\name{abba_submit_k8s_job_local}
\alias{abba_submit_k8s_job_local}
\title{Submit an R program for execution on a Kubernetes cluster}
\usage{
abba_submit_k8s_job_local(
  file_path,
  batch_group_id = "",
  user_tag = "",
  cpu_limit = 1L,
  memory_limit = "512M",
  container = getOption("abba.default.container"),
  mounts = "",
  auto_mount_home = FALSE,
  home_nfs_ip_address = getOption("abba.home.nfs.ip.address"),
  namespace = getOption("abba.k8s.namespace"),
  username = NULL
)
}
\arguments{
\item{file_path}{Full path to R file}

\item{batch_group_id}{Group ID for batch processing}

\item{user_tag}{Optional; a string that describes what kind of job will be scheduled to run}

\item{cpu_limit}{Maximum number of cores available for Kubernetes container}

\item{memory_limit}{Maximum amount of RAM available for Kubernetes container}

\item{container}{A valid container image name provided as a character string. Defaults to the option abba.default.container.}

\item{mounts}{Specifically formatted list with information bout volumes that container would have access to during the run}

\item{auto_mount_home}{set to TRUE to mount service user home directory}

\item{home_nfs_ip_address}{IP address for mounting service user home directory}

\item{namespace}{Kubernetes namespace to put the job in}

\item{username}{user whose authority will be used to run the program}
}
\value{
A list with job_id and batch_id attributes in case of successful submission
}
\description{
Submit an R program for execution on a Kubernetes cluster
}
\examples{
\dontrun{
job_info <- abba_submit_k8s_job_local("path/to/your/program.R", batch_group_id='SDTM')
}

}
