% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/submit_job_workbench.R
\name{abba_rslauncher_get_job_status_local}
\alias{abba_rslauncher_get_job_status_local}
\title{Get Workbench job status for a given vector/list of job IDs}
\usage{
abba_rslauncher_get_job_status_local(job_ids, ...)
}
\arguments{
\item{job_ids}{A list/vector of job IDs}

\item{...}{other positional/keyword arguments that will be ignored}
}
\value{
a vector of job ID statuses
}
\description{
Get Workbench job status for a given vector/list of job IDs
}
\examples{
\dontrun{
job_statuses <- abba_rslauncher_get_job_status_local(c('job-id-1', 'job-id-2'))
}
}
