% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/submit_job_k8s.R
\name{abba_get_k8s_batch_status_local}
\alias{abba_get_k8s_batch_status_local}
\title{Get status of all jobs in a batch}
\usage{
abba_get_k8s_batch_status_local(
  batch_id,
  namespace = getOption("abba.k8s.namespace")
)
}
\arguments{
\item{batch_id}{unique identifier for a batch}

\item{namespace}{Kubernetes namespace}
}
\value{
list of statuses for every job in a batch
}
\description{
Get status of all jobs in a batch
}
\examples{
\dontrun{
status <- abba_get_k8s_batch_status_local("batch-sdtm-abc123")
}
}
