% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/submit_job_slurm.R
\name{abba_slurm_get_job_log}
\alias{abba_slurm_get_job_log}
\title{Return job log for each submitted job ID}
\usage{
abba_slurm_get_job_log(job_ids, ...)
}
\arguments{
\item{job_ids}{valid SLURM job IDs}

\item{...}{other parameters that will be ignored}
}
\value{
list of character vectors. each character vector would represent program log.
}
\description{
Return job log for each submitted job ID
}
\examples{
\dontrun{
job_log <- abba_slurm_get_job_log("156")}

}
