% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api_requests.R
\name{abba_get_batch_log}
\alias{abba_get_batch_log}
\title{Send GET request to get logs of all jobs in a batch}
\usage{
abba_get_batch_log(
  batch_id,
  api_address = Sys.getenv("ABBA_API_ADDRESS"),
  api_key = Sys.getenv("ABBA_API_KEY")
)
}
\arguments{
\item{batch_id}{unique batch identificator}

\item{api_address}{URL to send requests to, hosted in Posit Connect. Defaults
to environment variable ABBA_API_ADDRESS.}

\item{api_key}{API Key for accessing restricted endpoints. Defaults to
environment variable ABBA_API_KEY}
}
\value{
body of request`s response in a list format
}
\description{
Send GET request to get logs of all jobs in a batch
}
\examples{
\dontrun{
response <- abba_get_job_log('1234j-13j4l5k-ajslfd')}
}
