% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api_requests.R
\name{abba_wait_for_job_log}
\alias{abba_wait_for_job_log}
\title{Monitor job status and retrieve its log when the job finishes running}
\usage{
abba_wait_for_job_log(
  job_id,
  poll_interval_seconds = 3,
  timeout_seconds = 600,
  api_address = Sys.getenv("ABBA_API_ADDRESS"),
  api_key = Sys.getenv("ABBA_API_KEY"),
  ...
)
}
\arguments{
\item{job_id}{unique job identificator}

\item{poll_interval_seconds}{Total time to wait before timeout in seconds}

\item{timeout_seconds}{Total time to wait before timeout in seconds}

\item{api_address}{URL to send requests to, hosted in Posit Connect. Defaults
to environment variable ABBA_API_ADDRESS.}

\item{api_key}{API Key for accessing restricted endpoints. Defaults to
environment variable ABBA_API_KEY}

\item{...}{Other arguments that will be ignored}
}
\value{
list with 2 attributes: job_id for submitted job`s id, and its logs
}
\description{
Monitor job status and retrieve its log when the job finishes running
}
\examples{
\dontrun{
response <- abba_wait_for_job_log('sdfj4-asdjlk-bjslk')}
}
