% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api_requests.R
\name{abba_get_job_log}
\alias{abba_get_job_log}
\title{Send GET request to get logs of specified Jobs}
\usage{
abba_get_job_log(
  job_ids,
  api_address = Sys.getenv("ABBA_API_ADDRESS"),
  api_key = Sys.getenv("ABBA_API_KEY")
)
}
\arguments{
\item{job_ids}{A list of job IDs to get logs for}

\item{api_address}{URL to send requests to, hosted in Posit Connect. Defaults
to environment variable ABBA_API_ADDRESS.}

\item{api_key}{API Key for accessing restricted endpoints. Defaults to
environment variable ABBA_API_KEY}
}
\value{
body of request`s response in a list format
}
\description{
Send GET request to get logs of specified Jobs
}
\examples{
\dontrun{
response <- abba_get_job_log('1234j-13j4l5k-ajslfd')}
}
