% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/submit_job_k8s.R
\name{submit_k8s_yaml}
\alias{submit_k8s_yaml}
\title{Submit a job profile for execution on a Kubernetes cluster}
\usage{
submit_k8s_yaml(yaml_full_path)
}
\arguments{
\item{yaml_full_path}{Path to YAML config file}
}
\value{
A string containing Job ID
}
\description{
Submit a job profile for execution on a Kubernetes cluster
}
\examples{
\dontrun{
job_id <- submit_k8s_yaml("/tmp/my_job.yaml")
}
}
