% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/submit_job_k8s.R
\name{abba_watch_k8s_job_local}
\alias{abba_watch_k8s_job_local}
\title{Watch a K8S job that has been submitted to Workbench, periodically polling it's execution status.}
\usage{
abba_watch_k8s_job_local(
  job_id = "",
  poll_interval_seconds = 3,
  timeout_seconds = 600,
  namespace = getOption("abba.k8s.namespace")
)
}
\arguments{
\item{job_id}{Job ID. Typically obtained as a return value from submit_job and similar functions}

\item{poll_interval_seconds}{Time interval for polling job status in seconds}

\item{timeout_seconds}{Total time to wait before timeout in seconds}

\item{namespace}{Kubernetes namespace}
}
\value{
a list of pods, their IDs and execution statuses
}
\description{
Watch a K8S job that has been submitted to Workbench, periodically polling it's execution status.
}
\examples{
\dontrun{
result <- abba_watch_k8s_job_local("job-sdtm-f0bf6848-46de-45b8-9fae-0e732b104760", 10, 3000)
}

}
