% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_dataset.R
\name{calculate_run_group}
\alias{calculate_run_group}
\title{Calculate run_group variable using inputs and outputs of programs supplied by user}
\usage{
calculate_run_group(x, col_name = "run_group")
}
\arguments{
\item{x}{input data frame. Must contain columns 'inputs', 'outputs' that list
input/output datasets for each program}

\item{col_name}{name of newly created variable. Defaults to 'run_group_calculated'}
}
\value{
an input data frame with one new column
}
\description{
Calculate run_group variable using inputs and outputs of programs supplied by user
}
\examples{
input_ds <- as.data.frame(list(program_name=c('prog1.R', 'prog2.R'),
                               inputs=c('ds0.xpt', 'ds1.xpt'),
                               outputs=c('ds1.xpt', 'ds2.xpt')))
batch_ready <- calculate_run_group
}
