% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api_requests.R
\name{abba_wait_for_batch_log}
\alias{abba_wait_for_batch_log}
\title{Monitor batch status and retrieve its log when the all jobs in batch finish
running}
\usage{
abba_wait_for_batch_log(
  batch_id,
  poll_interval_seconds = 3,
  timeout_seconds = 600,
  api_address = Sys.getenv("ABBA_API_ADDRESS"),
  api_key = Sys.getenv("ABBA_API_KEY")
)
}
\arguments{
\item{batch_id}{unique batch identificator}

\item{poll_interval_seconds}{Total time to wait before timeout in seconds}

\item{timeout_seconds}{Total time to wait before timeout in seconds}

\item{api_address}{URL to send requests to, hosted in Posit Connect. Defaults
to environment variable ABBA_API_ADDRESS.}

\item{api_key}{API Key for accessing restricted endpoints. Defaults to
environment variable ABBA_API_KEY}
}
\value{
list with 2 sublists: job_ids and their logs
}
\description{
Monitor batch status and retrieve its log when the all jobs in batch finish
running
}
\examples{
\dontrun{
response <- abba_wait_for_batch_log('batch-sdtm-sdfj4-asdjlk-bjslk')}
}
