% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_job_log.R
\name{abba_get_k8s_job_log_local}
\alias{abba_get_k8s_job_log_local}
\title{Get log for every job specified in an input vector/list}
\usage{
abba_get_k8s_job_log_local(
  job_ids,
  namespace = getOption("abba.k8s.namespace")
)
}
\arguments{
\item{job_ids}{A list of job IDs to get logs for}

\item{namespace}{Kubernetes namespace to search for job}
}
\value{
A list of job logs. Each list entry will contain complete log for a job
}
\description{
Get log for every job specified in an input vector/list
}
\examples{
\dontrun{
logs <- abba_get_k8s_job_log_local(c("job-sdtm-abc123", "job-adam-def456"))
}
}
