% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/submit_job_workbench.R
\name{abba_rslauncher_submit_job_local}
\alias{abba_rslauncher_submit_job_local}
\title{Create a job for executing an R program}
\usage{
abba_rslauncher_submit_job_local(p, log_path, user_tag = "", ...)
}
\arguments{
\item{p}{path to program}

\item{log_path}{Path to the directory where the log will be saved.
Required; must be supplied explicitly so that logs are never written to an
unexpected location in the user's filespace.}

\item{user_tag}{optional; any user tags user might want to add to the job}

\item{...}{other arguments that will be passed to internal rslauncher_submit_job function}
}
\value{
job id
}
\description{
Create a job for executing an R program
}
\examples{
\dontrun{
job_id <- abba_rslauncher_submit_job_local("/path/to/program.R",
                                           log_path = "/path/to/logs")
}
}
