% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/submit_job_slurm.R
\name{abba_slurm_get_job_succeeded}
\alias{abba_slurm_get_job_succeeded}
\title{Check whether SLURM jobs have been fully executed.}
\usage{
abba_slurm_get_job_succeeded(job_ids, ...)
}
\arguments{
\item{job_ids}{a list/vector of  valid SLURM job IDs}

\item{...}{other parameters that will be ignored}
}
\value{
a named boolean vector. TRUE if job finished running and has 'COMPLETED' status, FALSE otherwise.
If job has not finished running, a NULL will be returned.
}
\description{
Check whether SLURM jobs have been fully executed.
}
\examples{
\dontrun{
job_statuses <- abba_slurm_get_job_succeeded(c('job-id-1', 'job-id-2'))
}
}
